@@ -29,6 +29,7 @@ from .functions_components import (
     get_comp_statistics,
     split_uid,
     get_all_enabled_comps_from_system,
+    get_comp_statistics_all_systems,
     )
 
 from .streamlit_functions import (
@@ -39,6 +40,6 @@ from .streamlit_functions import (
 
 # deprecated functions
 from ._backend_calls import _backend_put, _backend_get, _backend_post
-
+from .functions_projects import update_sys_generator
 # renamed functions
 from .renamed_functions import get_mission_info, get_all_decisions, translate_tomlsystem_to_backend, title_with_project_selector, load_project
\ No newline at end of file
