@@ -1,21 +1,28 @@
 import ast
 import json
 from pathlib import Path
-import toml
+
 import pandas as pd
-from .logging_setup import logger_ai4cehelper
-from ._backend_calls import _backend_POST, _backend_PUT, _backend_GET
+import warnings
+import toml
+
+from ._backend_calls import _backend_GET
+from ._backend_calls import _backend_POST
+from ._backend_calls import _backend_PUT
 from .functions import load_file
+from .logging_setup import logger_ai4cehelper
+from deprecated import deprecated
 
 def create_new_project(project_info: dict):
     """Create a new project in the backend.
     Params:
         project_info(dict):
     Returns:
-        dict:
+        dict: Backend version of the project.
+    Raises:
+        Exception: If there is an error creating the new project.
     """
-    status_code, project = _backend_POST(
-        endpoint=f"/v2/projects/", data=project_info)
+    status_code, project = _backend_POST(endpoint=f"/v2/projects/", data=project_info)
     if status_code != 201:
         logger_ai4cehelper.error(f"(cnp) Error creating new project ({status_code}): {project}")
         raise Exception(f"Error creating new project ({status_code}): {project}")
@@ -27,11 +34,11 @@ def get_list_of_all_project_infos() -> list[tuple[int, str, str]]:
     """A function to connect to the backend and get a list of all projects
     in the current project data base.
 
-    Params:
-        None
-
     Returns:
         tuple[int, str, str]: A list of tuples containing project ID, project name, and last modified date.
+    
+    Raises:
+        Exception: If there is an error fetching the list of projects.
     """
 
     status_code, projects = _backend_GET(endpoint=f"/v2/projects/")
@@ -44,6 +51,7 @@ def get_list_of_all_project_infos() -> list[tuple[int, str, str]]:
         logger_ai4cehelper.error(f"(gloapi) Error fetching projects ({status_code}): {projects}")
         raise Exception(f"Error fetching projects ({status_code}): {projects}")
 
+
 def get_project_info(project_id: int) -> dict:
     status_code, project_info = _backend_GET(endpoint=f"/v2/projects/{project_id}/")
     if status_code != 200:
@@ -56,10 +64,13 @@ def set_project_name(project_id: int, project_name: str) -> dict:
     """A function to set the project name of a specific project, identified by its ID.
 
     Params:
-        project_id (int): The project Identification Number
-        project_name(str): Name of the project/mission, eg OPS-Sat
+        project_id   (int): The project ID
+        project_name (str): Name of the project/mission, eg OPS-Sat
     Returns:
-        dict: json response from backend
+        dict: Backend version of the general project info.
+    Raises:
+        KeyError: If the project is not found.
+        Exception: If there is an error setting the project name.
     """
 
     data = {
@@ -70,12 +81,22 @@ def set_project_name(project_id: int, project_name: str) -> dict:
     # status, msg = _backend_put(endpoint=f"/v2/projects/{project_id}", data=data)
     # return (status, msg)
     status_code, response = _backend_PUT(endpoint=f"/v2/projects/{project_id}/", data=data)
+    if status_code == 404:
+        logger_ai4cehelper.error(f"(spn) Project with ID {project_id} not found.")
+        raise KeyError(f"Project with ID {project_id} not found.")
     if status_code != 200:
-        logger_ai4cehelper.error(f"(spn) Error setting project name: {response}")
-        return {status_code: response}
+        logger_ai4cehelper.error(f"(spn) Error setting project name ({status_code}): {response}")
+        raise Exception(f"Error setting project name ({status_code}): {response}")
     return response
 
+
 def get_recent_project() -> tuple[int, str, str]:
+    """A function to get the most recently modified project from the backend.
+    Returns:
+        tuple[int, str, str]: A tuple containing project ID, project name, and last modified date.
+    Raises:
+        Exception: If there is an error fetching the recent project.
+    """
     status_code, response = _backend_GET(endpoint=f"/v2/projects/recent/")
     if status_code != 200:
         logger_ai4cehelper.error(f"(grp) Error fetching recent project ({status_code}): {response}")
@@ -86,13 +107,21 @@ def get_recent_project() -> tuple[int, str, str]:
 def get_mission_orbit(project_id: int) -> dict:
     """A function to get the mission orbit info for a specific project based on its ID.
     Params:
-        project_id (int): The project Identification Number
+        project_id (int): The project ID
 
     Returns:
         dict: Keplerian elements to define the orbit
+
+    Raises:
+        KeyError: If no mission is found for the given project ID.
+        KeyError: If no orbit information is found in the mission data.
+        Exception: If there is another error fetching the mission orbit.
     """
 
     status_code, response = _backend_GET(endpoint=f"/v2/projects/{project_id}/mission/")
+    if status_code == 404:
+        logger_ai4cehelper.error(f"(gmo) No mission orbit found for project {project_id}.")
+        raise KeyError(f"No mission orbit found for project {project_id}.")
     if status_code != 200:
         logger_ai4cehelper.error(f"(gmo) Error fetching mission orbit ({status_code}): {response}")
         raise Exception(f"Error fetching mission orbit ({status_code}): {response}")
@@ -102,23 +131,25 @@ def get_mission_orbit(project_id: int) -> dict:
     return response["orbit"]
 
 
-
 def set_mission_orbit(project_id: int, orbit_info: dict) -> dict:
     """A function to set the orbit in the project database. Will create an empty mission parent if it does not exist.
     Params:
-        project_id (int): The project Identification Number
+        project_id (int): The project ID
         orbit_info(dict): Information about the satellite's orbit
+
     Returns:
-        dict: 
-     """
+        dict: Backend version of the mission orbit or the status_code and error message.
+
+    Raises:
+        Exception: If there is an error creating or updating the mission orbit.
+    """
     # check if project exists
     status_code, project_info = _backend_GET(endpoint=f"/v2/projects/{project_id}/")
     if status_code != 200:
         logger_ai4cehelper.error(f"(smo) Error fetching project info ({status_code}): {project_info}")
         raise Exception(f"Error fetching project info ({status_code}): {project_info}")
-    
-    status_code, mission_info = _backend_GET(
-        endpoint=f"/v2/projects/{project_id}/mission/")
+
+    status_code, mission_info = _backend_GET(endpoint=f"/v2/projects/{project_id}/mission/")
     if status_code == 404:
         logger_ai4cehelper.info(f"(smo) No mission info found for project {project_id}. Creating new mission info.")
         mission_info = {
@@ -152,12 +183,12 @@ def get_enabled_components(nested_dict, enabled_components=None) -> list:
     Recursively traverses a nested dictionary to find and return a list of components that are enabled.
 
     Parameters:
-    nested_dict (dict): The nested dictionary to traverse.
-    enabled_components (list, optional): A list to store the names of the enabled components.
+        nested_dict (dict): The nested dictionary to traverse.
+        enabled_components (list, optional): A list to store the names of the enabled components.
                                          Defaults to None, in which case a new list is created.
 
     Returns:
-    list: A list of the names of the enabled components.
+        list: A list of the names of the enabled components.
     """
 
     if enabled_components is None:
@@ -165,7 +196,7 @@ def get_enabled_components(nested_dict, enabled_components=None) -> list:
 
     for key, value in nested_dict.items():
         if isinstance(value, dict):
-            if value.get('Enabled') == True:
+            if value.get("Enabled") == True:
                 enabled_components.append(key)
                 pass
             get_enabled_components(value, enabled_components)
@@ -173,8 +204,6 @@ def get_enabled_components(nested_dict, enabled_components=None) -> list:
     return enabled_components
 
 
-
-
 def traverse_and_modify(d: dict, sys_config_enabled: dict):
     # check for refactoring
     for key, value in d.items():
@@ -186,9 +215,13 @@ def traverse_and_modify(d: dict, sys_config_enabled: dict):
             # if d["Enabled"] is True or d["Enable"] is True or :
             try:
                 if d["Enabled"] is True:
-                    sys_config_enabled.update(enable_component(
-                        component_name=key, data=load_file(Path("data/backend_sys_default.json"))))
-            except KeyError as e: # what are you doing here -.-
+                    sys_config_enabled.update(
+                        enable_component(
+                            component_name=key,
+                            data=load_file(Path("data/backend_sys_default.json")),
+                        )
+                    )
+            except KeyError as e:  # what are you doing here -.-
                 print("Error: ", e)
 
 
@@ -214,34 +247,37 @@ def enable_component(component_name: str, data: dict) -> dict:
                 if isinstance(item, dict):
                     enable_component(component_name, item)
         if key.lower() == component_name.lower() or str(value).lower() == component_name.lower():
-            data['enabled'] = True
+            data["enabled"] = True
     return data
 
 
 def set_sys_arch(project_id: int, sys_arch: dict) -> dict:
     """A function to set the system architecture in the project database. Updates the existing architecture if it exists.
     Params:
-        project_id (int): The project Identification Number
-        sys_arch(dict): Information about the satelllite's modules, which form the system architecture
+        project_id (int): The project ID
+        sys_arch (dict): Information about the satellite's modules, which form the system architecture
     Returns:
-        dict: Backend version of the system architecture or the status_code and error message.
-     """
+        dict: Backend version of the system architecture.
+    Raises:
+        Exception: If there is an error creating or updating the system architecture.
+    """
 
-    status_code, system = _backend_GET(
-        endpoint=f"/v2/projects/{project_id}/system/")
+    status_code, system = _backend_GET(endpoint=f"/v2/projects/{project_id}/system/")
     if status_code == 404:
-        logger_ai4cehelper.info(f"(ssa) No system architecture found for project {project_id}. Creating new system architecture.")
-        status_code, response = _backend_POST(
-            endpoint=f"/v2/projects/{project_id}/system/", data=sys_arch)
+        logger_ai4cehelper.info(
+            f"(ssa) No system architecture found for project {project_id}. Creating new system architecture."
+        )
+        status_code, response = _backend_POST(endpoint=f"/v2/projects/{project_id}/system/", data=sys_arch)
         if status_code != 201:
             logger_ai4cehelper.error(f"(ssa) Error creating system architecture ({status_code}): {response}")
             raise Exception(f"Error creating system architecture ({status_code}): {response}")
         return response
     elif status_code == 200:
-        logger_ai4cehelper.info(f"(ssa) System architecture found for project {project_id}. Updating system architecture.")
+        logger_ai4cehelper.info(
+            f"(ssa) System architecture found for project {project_id}. Updating system architecture."
+        )
         system.update(sys_arch)
-        status_code, response = _backend_PUT(
-            endpoint=f"/v2/projects/{project_id}/system/", data=system)
+        status_code, response = _backend_PUT(endpoint=f"/v2/projects/{project_id}/system/", data=system)
         if status_code != 200:
             logger_ai4cehelper.error(f"(ssa) Error updating system architecture ({status_code}): {response}")
             raise Exception(f"Error updating system architecture ({status_code}): {response}")
@@ -251,61 +287,76 @@ def set_sys_arch(project_id: int, sys_arch: dict) -> dict:
 def get_sys_arch(project_id: int) -> dict:
     """A function to get the system configuration info for a specific project based on its ID.
     Params:
-        project_id (int): The project Identification Number
+        project_id (int): The project ID
 
     Returns:
-        dict: System information
+        dict: Backend version of the system architecture.
+
+    Raises:
+        KeyError: If no system architecture is found for the given project ID.
+        Exception: If there is another error fetching the system architecture.
     """
 
-    status_code, response = _backend_GET(
-        endpoint=f"/v2/projects/{project_id}/system/")
+    status_code, response = _backend_GET(endpoint=f"/v2/projects/{project_id}/system/")
+    if status_code == 404:
+        logger_ai4cehelper.error(f"(gsa) No system architecture found for project {project_id}.")
+        raise KeyError(f"No system architecture found for project {project_id}.")
     if status_code != 200:
         logger_ai4cehelper.error(f"(gsa) Error fetching system architecture ({status_code}): {response}")
         raise Exception(f"Error fetching system architecture ({status_code}): {response}")
     return response
 
+
 #  set_comp_list(project: int, not_yet_defined: dict)
 # For each selected system generator, we need a place to store the corresponding found comp lists
 # Every generator can produce n sets of components
+@deprecated(reason="update_sys_generator() is deprecated, use set_sys_generator() instead.", version="0.10.12")
+def update_sys_generator(project_id: int, sys_gen_info: dict) -> dict:
+    """Deprecated: update_sys_generator is deprecated, use set_sys_generator instead."""
+    warnings.warn("update_sys_generator is deprecated, use set_sys_generator instead.", DeprecationWarning)
+    return set_sys_generator(project_id=project_id, sys_gen_info=sys_gen_info)
 
 
 def set_sys_generator(project_id: int, sys_gen_info: dict) -> dict:
-    status_code, response = _backend_POST(
-        endpoint=f"/v2/projects/{project_id}/sysgen/", data=sys_gen_info)
+    """A function to set the system generator info in the project database. Updates the existing generator if it exists.
+    Params:
+        project_id (int): The project ID 
+        sys_gen_info (dict): Information about the system generator
+    Returns:
+        dict: Backend version of the system generator info.
+    Raises:
+        Exception: If there is an error creating or updating the system generator info.
+    """
+    if "id" in sys_gen_info:
+        status_code, existing_sys_gen = _backend_GET(endpoint=f"/v2/projects/{project_id}/sysgen/{sys_gen_info['id']}/")
+        if status_code == 200:
+            logger_ai4cehelper.info(
+                f"(ssg) System generator info found for project {project_id}. Updating system generator info."
+            )
+            existing_sys_gen.update(sys_gen_info)
+            status_code, response = _backend_PUT(
+                endpoint=f"/v2/projects/{project_id}/sysgen/{sys_gen_info['id']}/",
+                data=existing_sys_gen,
+            )
+            if status_code != 200:
+                logger_ai4cehelper.error(f"(ssg) Error updating system generator info ({status_code}): {response}")
+                raise Exception(f"Error updating system generator info ({status_code}): {response}")
+            return response
+    logger_ai4cehelper.info(f"(ssg) Creating new system generator info for project {project_id}.")
+    status_code, response = _backend_POST(endpoint=f"/v2/projects/{project_id}/sysgen/", data=sys_gen_info)
     if status_code != 201:
         logger_ai4cehelper.error(f"(ssg) Error creating system generator info ({status_code}): {response}")
         raise Exception(f"Error creating system generator info ({status_code}): {response}")
-    return status_code, response
+    return response
 
 
-def update_sys_generator(project_id: int, sys_gen_info: dict) -> dict:
+def set_trained_model(
+    project_id: int,
+    sysgen_id: int,
+    environment_settings: dict = {},
+    model_file: bytes = None,
+) -> dict:
 
-    if "id" not in sys_gen_info:
-        logger_ai4cehelper.error(f"(usg) sys_gen_info must contain an 'id' field for updating.")
-        raise KeyError("sys_gen_info must contain an 'id' field for updating.")
-
-    status_code, sys_gen = _backend_GET(
-        endpoint=f"/v2/projects/{project_id}/sysgen/")
-    if status_code == 404:
-        logger_ai4cehelper.info(f"(usg) No system generator info found for project {project_id}. Creating new system generator info.")
-        status_code, response = _backend_POST(
-            endpoint=f"/v2/projects/{project_id}/sysgen/", data=sys_gen_info)
-        if status_code != 201:
-            logger_ai4cehelper.error(f"(usg) Error creating system generator info ({status_code}): {response}")
-            raise Exception(f"Error creating system generator info ({status_code}): {response}")
-        return response
-    elif status_code == 200:
-        logger_ai4cehelper.info(f"(usg) System generator info found for project {project_id}. Updating system generator info.")
-        sys_gen.update(sys_gen_info)
-        status_code, response = _backend_PUT(
-            endpoint=f"/v2/projects/{project_id}/sysgen/{sys_gen_info['id']}/", data=sys_gen)
-        if status_code != 200:
-            logger_ai4cehelper.error(f"(usg) Error updating system generator info ({status_code}): {response}")
-            raise Exception(f"Error updating system generator info ({status_code}): {response}")
-        return response
-
-def set_trained_model(project_id: int, sysgen_id: int, environment_settings: dict = {}, model_file: bytes = None) -> dict:
-   
     if not model_file and not environment_settings:
         logger_ai4cehelper.error(f"(stm) Either environment_settings or model_file must be provided.")
         raise ValueError("Either environment_settings or model_file must be provided.")
@@ -318,20 +369,27 @@ def set_trained_model(project_id: int, sysgen_id: int, environment_settings: dic
     sysgen_db = update_sys_generator(project_id=project_id, sys_gen_info=sys_gen)
 
     if model_file:
-        status_code, response = _backend_GET(
-            endpoint=f"/v2/projects/{project_id}/sysgen/{sysgen_id}/model/")
+        status_code, response = _backend_GET(endpoint=f"/v2/projects/{project_id}/sysgen/{sysgen_id}/model/")
         if status_code == 404:
-            logger_ai4cehelper.info(f"(stm) No trained model found for sysgen {sysgen_id} in project {project_id}. Creating new trained model.")
+            logger_ai4cehelper.info(
+                f"(stm) No trained model found for sysgen {sysgen_id} in project {project_id}. Creating new trained model."
+            )
             status_code, response = _backend_POST(
-                endpoint=f"/v2/projects/{project_id}/sysgen/{sysgen_id}/model/", data={"model_file": model_file})
+                endpoint=f"/v2/projects/{project_id}/sysgen/{sysgen_id}/model/",
+                data={"model_file": model_file},
+            )
             if status_code != 201:
                 logger_ai4cehelper.error(f"(stm) Error creating trained model ({status_code}): {response}")
                 raise Exception(f"Error creating trained model ({status_code}): {response}")
             return response
         elif status_code == 200:
-            logger_ai4cehelper.info(f"(stm) Trained model found for sysgen {sysgen_id} in project {project_id}. Updating trained model.")
+            logger_ai4cehelper.info(
+                f"(stm) Trained model found for sysgen {sysgen_id} in project {project_id}. Updating trained model."
+            )
             status_code, response = _backend_PUT(
-                endpoint=f"/v2/projects/{project_id}/sysgen/{sysgen_id}/model/", data={"model_file": model_file})
+                endpoint=f"/v2/projects/{project_id}/sysgen/{sysgen_id}/model/",
+                data={"model_file": model_file},
+            )
             if status_code != 200:
                 logger_ai4cehelper.error(f"(stm) Error updating trained model ({status_code}): {response}")
                 raise Exception(f"Error updating trained model ({status_code}): {response}")
@@ -339,17 +397,17 @@ def set_trained_model(project_id: int, sysgen_id: int, environment_settings: dic
         raise Exception(f"(stm) Unexpected status code ({status_code}): {response}")
     return sysgen_db
 
+
 def get_prepared_system_generator_info(project_id: int) -> list:
     """A function to get all prepared system generators.
     Params:
-        project_id (int): The project Identification Number
+        project_id (int): The project ID
 
     Returns:
         list: list of dictionaries with infos for the prepared system generators
     """
 
-    status_code, response = _backend_GET(
-        endpoint=f"/v2/projects/{project_id}/sysgen/")
+    status_code, response = _backend_GET(endpoint=f"/v2/projects/{project_id}/sysgen/")
     if status_code != 200:
         logger_ai4cehelper.error(f"(gpsg) Error fetching prepared system generators ({status_code}): {response}")
         raise Exception(f"Error fetching prepared system generators ({status_code}): {response}")
@@ -372,49 +430,49 @@ def set_sys_arch_defaults(system: dict, project_id: int) -> dict:
         "OAP_W": 0,
         "requirements": {},
         "aodcs": {
-          "enabled": False,
-          "amount_of_entities": 0,
-          "description": "string",
-          "eFunctions": [],
-          "type": "string",
-          "components": [],
-          "depending_parameters": [],
-          "cmg": {
             "enabled": False,
             "amount_of_entities": 0,
             "description": "string",
             "eFunctions": [],
-            "requirements": {}
-          },
-          "magnetorquers": {
-            "enabled": False,
-            "amount_of_entities": 0,
-            "description": "string",
-            "eFunctions": [],
-            "requirements": {}
-          },
+            "type": "string",
+            "components": [],
+            "depending_parameters": [],
+            "cmg": {
+                "enabled": False,
+                "amount_of_entities": 0,
+                "description": "string",
+                "eFunctions": [],
+                "requirements": {},
+            },
+            "magnetorquers": {
+                "enabled": False,
+                "amount_of_entities": 0,
+                "description": "string",
+                "eFunctions": [],
+                "requirements": {},
+            },
         },
         "tcs": {
-          "enabled": False,
-          "amount_of_entities": 0,
-          "description": "string",
-          "eFunctions": [],
-          "requirements": {},
-          "coatings": {
             "enabled": False,
             "amount_of_entities": 0,
             "description": "string",
             "eFunctions": [],
-            "requirements": {}
-          },
-          "heaters": {
-            "enabled": False,
-            "amount_of_entities": 0,
-            "description": "string",
-            "eFunctions": [],
-            "requirements": {}
-          }
-        }
+            "requirements": {},
+            "coatings": {
+                "enabled": False,
+                "amount_of_entities": 0,
+                "description": "string",
+                "eFunctions": [],
+                "requirements": {},
+            },
+            "heaters": {
+                "enabled": False,
+                "amount_of_entities": 0,
+                "description": "string",
+                "eFunctions": [],
+                "requirements": {},
+            },
+        },
     }
 
     sys_config_enabled = {}
@@ -428,20 +486,38 @@ def set_sys_arch_defaults(system: dict, project_id: int) -> dict:
     return response
 
 
-def set_new_project(project_info: dict) -> dict:
-    """Create a new project in the backend.
+def set_new_project(project_info: dict) -> tuple[int, dict]:
+    """Create a new project in the backend. Updates the project if it already exists.
     Params:
-        project_info(dict):
+        project_info(dict): Information about the project to create or update.
     Returns:
-        dict:
+        tuple[int, dict]: Status code and backend version of the project.
+    Raises:
+        Exception: If there is an error creating the new project.
     """
-    status_code, response = _backend_POST(
-        endpoint="/v2/projects/", data=project_info)
+    if "id" in project_info:
+        status_code, existing_project = _backend_GET(endpoint=f"/v2/projects/{project_info['id']}/")
+        if status_code == 200:
+            logger_ai4cehelper.info(
+                f"(snp) Project with ID {project_info['id']} found. Updating existing project."
+            )
+            existing_project.update(project_info)
+            status_code, response = _backend_PUT(
+                endpoint=f"/v2/projects/{project_info['id']}/",
+                data=existing_project,
+            )
+            if status_code != 200:
+                logger_ai4cehelper.error(f"(snp) Error updating project ({status_code}): {response}")
+                raise Exception(f"Error updating project ({status_code}): {response}")
+            return status_code, response
+    logger_ai4cehelper.info(f"(snp) Creating new project.")
+    status_code, response = _backend_POST(endpoint=f"/v2/projects/", data=project_info)
     if status_code != 201:
         logger_ai4cehelper.error(f"(snp) Error creating new project ({status_code}): {response}")
         raise Exception(f"Error creating new project ({status_code}): {response}")
     return status_code, response
 
+
 # def get_trained_model(project_id: int) -> mode: ASKYOUNES
 #     pass
 
@@ -451,4 +527,3 @@ def set_new_project(project_info: dict) -> dict:
 
 
 # def get_train_logs(project_id: int) -> logs: ASKYOUNE   pass
-
