@@ -2,14 +2,18 @@ import logging
 import os
 import sys
 
-LOG_PATH = os.environ.get("LOG_PATH", "/var/log/ai4ce-helper")
+LOG_PATH = os.environ.get("LOG_PATH", "logs")
+
+# create log directory if it does not exist
+if not os.path.exists(LOG_PATH):
+    os.makedirs(LOG_PATH)
 
 # Logging
 
 # log_format = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
 log_format = logging.Formatter("%(levelname)s\t[%(name)s]\t%(message)s")
 
-logger_ai4cehelper = logging.getLogger("cdf_benchmark")
+logger_ai4cehelper = logging.getLogger("ai4ce_helper")
 logger_ai4cehelper.setLevel(logging.DEBUG)
 logger_ai4cehelper.propagate = False
 
